# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import os
import re
from typing import (
    Dict,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
    cast,
)
import warnings

from google.api_core import client_options as client_options_lib
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.exceptions import MutualTLSChannelError  # type: ignore
from google.auth.transport import mtls  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.apps.events_subscriptions_v1 import gapic_version as package_version

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object, None]  # type: ignore

from google.api_core import operation  # type: ignore
from google.api_core import operation_async  # type: ignore
from google.longrunning import operations_pb2  # type: ignore
from google.protobuf import duration_pb2  # type: ignore
from google.protobuf import empty_pb2  # type: ignore
from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore

from google.apps.events_subscriptions_v1.services.subscriptions_service import pagers
from google.apps.events_subscriptions_v1.types import (
    subscription_resource,
    subscriptions_service,
)

from .transports.base import DEFAULT_CLIENT_INFO, SubscriptionsServiceTransport
from .transports.grpc import SubscriptionsServiceGrpcTransport
from .transports.grpc_asyncio import SubscriptionsServiceGrpcAsyncIOTransport
from .transports.rest import SubscriptionsServiceRestTransport


class SubscriptionsServiceClientMeta(type):
    """Metaclass for the SubscriptionsService client.

    This provides class-level methods for building and retrieving
    support objects (e.g. transport) without polluting the client instance
    objects.
    """

    _transport_registry = (
        OrderedDict()
    )  # type: Dict[str, Type[SubscriptionsServiceTransport]]
    _transport_registry["grpc"] = SubscriptionsServiceGrpcTransport
    _transport_registry["grpc_asyncio"] = SubscriptionsServiceGrpcAsyncIOTransport
    _transport_registry["rest"] = SubscriptionsServiceRestTransport

    def get_transport_class(
        cls,
        label: Optional[str] = None,
    ) -> Type[SubscriptionsServiceTransport]:
        """Returns an appropriate transport class.

        Args:
            label: The name of the desired transport. If none is
                provided, then the first transport in the registry is used.

        Returns:
            The transport class to use.
        """
        # If a specific transport is requested, return that one.
        if label:
            return cls._transport_registry[label]

        # No transport is requested; return the default (that is, the first one
        # in the dictionary).
        return next(iter(cls._transport_registry.values()))


class SubscriptionsServiceClient(metaclass=SubscriptionsServiceClientMeta):
    """A service that manages subscriptions to Google Workspace
    events.
    """

    @staticmethod
    def _get_default_mtls_endpoint(api_endpoint):
        """Converts api endpoint to mTLS endpoint.

        Convert "*.sandbox.googleapis.com" and "*.googleapis.com" to
        "*.mtls.sandbox.googleapis.com" and "*.mtls.googleapis.com" respectively.
        Args:
            api_endpoint (Optional[str]): the api endpoint to convert.
        Returns:
            str: converted mTLS api endpoint.
        """
        if not api_endpoint:
            return api_endpoint

        mtls_endpoint_re = re.compile(
            r"(?P<name>[^.]+)(?P<mtls>\.mtls)?(?P<sandbox>\.sandbox)?(?P<googledomain>\.googleapis\.com)?"
        )

        m = mtls_endpoint_re.match(api_endpoint)
        name, mtls, sandbox, googledomain = m.groups()
        if mtls or not googledomain:
            return api_endpoint

        if sandbox:
            return api_endpoint.replace(
                "sandbox.googleapis.com", "mtls.sandbox.googleapis.com"
            )

        return api_endpoint.replace(".googleapis.com", ".mtls.googleapis.com")

    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = "workspaceevents.googleapis.com"
    DEFAULT_MTLS_ENDPOINT = _get_default_mtls_endpoint.__func__(  # type: ignore
        DEFAULT_ENDPOINT
    )

    _DEFAULT_ENDPOINT_TEMPLATE = "workspaceevents.{UNIVERSE_DOMAIN}"
    _DEFAULT_UNIVERSE = "googleapis.com"

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            SubscriptionsServiceClient: The constructed client.
        """
        credentials = service_account.Credentials.from_service_account_info(info)
        kwargs["credentials"] = credentials
        return cls(*args, **kwargs)

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            SubscriptionsServiceClient: The constructed client.
        """
        credentials = service_account.Credentials.from_service_account_file(filename)
        kwargs["credentials"] = credentials
        return cls(*args, **kwargs)

    from_service_account_json = from_service_account_file

    @property
    def transport(self) -> SubscriptionsServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            SubscriptionsServiceTransport: The transport used by the client
                instance.
        """
        return self._transport

    @staticmethod
    def subscription_path(
        subscription: str,
    ) -> str:
        """Returns a fully-qualified subscription string."""
        return "subscriptions/{subscription}".format(
            subscription=subscription,
        )

    @staticmethod
    def parse_subscription_path(path: str) -> Dict[str, str]:
        """Parses a subscription path into its component segments."""
        m = re.match(r"^subscriptions/(?P<subscription>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def topic_path(
        project: str,
        topic: str,
    ) -> str:
        """Returns a fully-qualified topic string."""
        return "projects/{project}/topics/{topic}".format(
            project=project,
            topic=topic,
        )

    @staticmethod
    def parse_topic_path(path: str) -> Dict[str, str]:
        """Parses a topic path into its component segments."""
        m = re.match(r"^projects/(?P<project>.+?)/topics/(?P<topic>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def user_path(
        user: str,
    ) -> str:
        """Returns a fully-qualified user string."""
        return "users/{user}".format(
            user=user,
        )

    @staticmethod
    def parse_user_path(path: str) -> Dict[str, str]:
        """Parses a user path into its component segments."""
        m = re.match(r"^users/(?P<user>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_billing_account_path(
        billing_account: str,
    ) -> str:
        """Returns a fully-qualified billing_account string."""
        return "billingAccounts/{billing_account}".format(
            billing_account=billing_account,
        )

    @staticmethod
    def parse_common_billing_account_path(path: str) -> Dict[str, str]:
        """Parse a billing_account path into its component segments."""
        m = re.match(r"^billingAccounts/(?P<billing_account>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_folder_path(
        folder: str,
    ) -> str:
        """Returns a fully-qualified folder string."""
        return "folders/{folder}".format(
            folder=folder,
        )

    @staticmethod
    def parse_common_folder_path(path: str) -> Dict[str, str]:
        """Parse a folder path into its component segments."""
        m = re.match(r"^folders/(?P<folder>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_organization_path(
        organization: str,
    ) -> str:
        """Returns a fully-qualified organization string."""
        return "organizations/{organization}".format(
            organization=organization,
        )

    @staticmethod
    def parse_common_organization_path(path: str) -> Dict[str, str]:
        """Parse a organization path into its component segments."""
        m = re.match(r"^organizations/(?P<organization>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_project_path(
        project: str,
    ) -> str:
        """Returns a fully-qualified project string."""
        return "projects/{project}".format(
            project=project,
        )

    @staticmethod
    def parse_common_project_path(path: str) -> Dict[str, str]:
        """Parse a project path into its component segments."""
        m = re.match(r"^projects/(?P<project>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_location_path(
        project: str,
        location: str,
    ) -> str:
        """Returns a fully-qualified location string."""
        return "projects/{project}/locations/{location}".format(
            project=project,
            location=location,
        )

    @staticmethod
    def parse_common_location_path(path: str) -> Dict[str, str]:
        """Parse a location path into its component segments."""
        m = re.match(r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)$", path)
        return m.groupdict() if m else {}

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[client_options_lib.ClientOptions] = None
    ):
        """Deprecated. Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """

        warnings.warn(
            "get_mtls_endpoint_and_cert_source is deprecated. Use the api_endpoint property instead.",
            DeprecationWarning,
        )
        if client_options is None:
            client_options = client_options_lib.ClientOptions()
        use_client_cert = os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false")
        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
        if use_client_cert not in ("true", "false"):
            raise ValueError(
                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
            )
        if use_mtls_endpoint not in ("auto", "never", "always"):
            raise MutualTLSChannelError(
                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
            )

        # Figure out the client cert source to use.
        client_cert_source = None
        if use_client_cert == "true":
            if client_options.client_cert_source:
                client_cert_source = client_options.client_cert_source
            elif mtls.has_default_client_cert_source():
                client_cert_source = mtls.default_client_cert_source()

        # Figure out which api endpoint to use.
        if client_options.api_endpoint is not None:
            api_endpoint = client_options.api_endpoint
        elif use_mtls_endpoint == "always" or (
            use_mtls_endpoint == "auto" and client_cert_source
        ):
            api_endpoint = cls.DEFAULT_MTLS_ENDPOINT
        else:
            api_endpoint = cls.DEFAULT_ENDPOINT

        return api_endpoint, client_cert_source

    @staticmethod
    def _read_environment_variables():
        """Returns the environment variables used by the client.

        Returns:
            Tuple[bool, str, str]: returns the GOOGLE_API_USE_CLIENT_CERTIFICATE,
            GOOGLE_API_USE_MTLS_ENDPOINT, and GOOGLE_CLOUD_UNIVERSE_DOMAIN environment variables.

        Raises:
            ValueError: If GOOGLE_API_USE_CLIENT_CERTIFICATE is not
                any of ["true", "false"].
            google.auth.exceptions.MutualTLSChannelError: If GOOGLE_API_USE_MTLS_ENDPOINT
                is not any of ["auto", "never", "always"].
        """
        use_client_cert = os.getenv(
            "GOOGLE_API_USE_CLIENT_CERTIFICATE", "false"
        ).lower()
        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto").lower()
        universe_domain_env = os.getenv("GOOGLE_CLOUD_UNIVERSE_DOMAIN")
        if use_client_cert not in ("true", "false"):
            raise ValueError(
                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
            )
        if use_mtls_endpoint not in ("auto", "never", "always"):
            raise MutualTLSChannelError(
                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
            )
        return use_client_cert == "true", use_mtls_endpoint, universe_domain_env

    @staticmethod
    def _get_client_cert_source(provided_cert_source, use_cert_flag):
        """Return the client cert source to be used by the client.

        Args:
            provided_cert_source (bytes): The client certificate source provided.
            use_cert_flag (bool): A flag indicating whether to use the client certificate.

        Returns:
            bytes or None: The client cert source to be used by the client.
        """
        client_cert_source = None
        if use_cert_flag:
            if provided_cert_source:
                client_cert_source = provided_cert_source
            elif mtls.has_default_client_cert_source():
                client_cert_source = mtls.default_client_cert_source()
        return client_cert_source

    @staticmethod
    def _get_api_endpoint(
        api_override, client_cert_source, universe_domain, use_mtls_endpoint
    ):
        """Return the API endpoint used by the client.

        Args:
            api_override (str): The API endpoint override. If specified, this is always
                the return value of this function and the other arguments are not used.
            client_cert_source (bytes): The client certificate source used by the client.
            universe_domain (str): The universe domain used by the client.
            use_mtls_endpoint (str): How to use the mTLS endpoint, which depends also on the other parameters.
                Possible values are "always", "auto", or "never".

        Returns:
            str: The API endpoint to be used by the client.
        """
        if api_override is not None:
            api_endpoint = api_override
        elif use_mtls_endpoint == "always" or (
            use_mtls_endpoint == "auto" and client_cert_source
        ):
            _default_universe = SubscriptionsServiceClient._DEFAULT_UNIVERSE
            if universe_domain != _default_universe:
                raise MutualTLSChannelError(
                    f"mTLS is not supported in any universe other than {_default_universe}."
                )
            api_endpoint = SubscriptionsServiceClient.DEFAULT_MTLS_ENDPOINT
        else:
            api_endpoint = SubscriptionsServiceClient._DEFAULT_ENDPOINT_TEMPLATE.format(
                UNIVERSE_DOMAIN=universe_domain
            )
        return api_endpoint

    @staticmethod
    def _get_universe_domain(
        client_universe_domain: Optional[str], universe_domain_env: Optional[str]
    ) -> str:
        """Return the universe domain used by the client.

        Args:
            client_universe_domain (Optional[str]): The universe domain configured via the client options.
            universe_domain_env (Optional[str]): The universe domain configured via the "GOOGLE_CLOUD_UNIVERSE_DOMAIN" environment variable.

        Returns:
            str: The universe domain to be used by the client.

        Raises:
            ValueError: If the universe domain is an empty string.
        """
        universe_domain = SubscriptionsServiceClient._DEFAULT_UNIVERSE
        if client_universe_domain is not None:
            universe_domain = client_universe_domain
        elif universe_domain_env is not None:
            universe_domain = universe_domain_env
        if len(universe_domain.strip()) == 0:
            raise ValueError("Universe Domain cannot be an empty string.")
        return universe_domain

    @staticmethod
    def _compare_universes(
        client_universe: str, credentials: ga_credentials.Credentials
    ) -> bool:
        """Returns True iff the universe domains used by the client and credentials match.

        Args:
            client_universe (str): The universe domain configured via the client options.
            credentials (ga_credentials.Credentials): The credentials being used in the client.

        Returns:
            bool: True iff client_universe matches the universe in credentials.

        Raises:
            ValueError: when client_universe does not match the universe in credentials.
        """

        default_universe = SubscriptionsServiceClient._DEFAULT_UNIVERSE
        credentials_universe = getattr(credentials, "universe_domain", default_universe)

        if client_universe != credentials_universe:
            raise ValueError(
                "The configured universe domain "
                f"({client_universe}) does not match the universe domain "
                f"found in the credentials ({credentials_universe}). "
                "If you haven't configured the universe domain explicitly, "
                f"`{default_universe}` is the default."
            )
        return True

    def _validate_universe_domain(self):
        """Validates client's and credentials' universe domains are consistent.

        Returns:
            bool: True iff the configured universe domain is valid.

        Raises:
            ValueError: If the configured universe domain is not valid.
        """
        self._is_universe_domain_valid = (
            self._is_universe_domain_valid
            or SubscriptionsServiceClient._compare_universes(
                self.universe_domain, self.transport._credentials
            )
        )
        return self._is_universe_domain_valid

    @property
    def api_endpoint(self):
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used by the client instance.
        """
        return self._universe_domain

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Optional[Union[str, SubscriptionsServiceTransport]] = None,
        client_options: Optional[Union[client_options_lib.ClientOptions, dict]] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the subscriptions service client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, SubscriptionsServiceTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that the ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client_options = client_options
        if isinstance(self._client_options, dict):
            self._client_options = client_options_lib.from_dict(self._client_options)
        if self._client_options is None:
            self._client_options = client_options_lib.ClientOptions()
        self._client_options = cast(
            client_options_lib.ClientOptions, self._client_options
        )

        universe_domain_opt = getattr(self._client_options, "universe_domain", None)

        (
            self._use_client_cert,
            self._use_mtls_endpoint,
            self._universe_domain_env,
        ) = SubscriptionsServiceClient._read_environment_variables()
        self._client_cert_source = SubscriptionsServiceClient._get_client_cert_source(
            self._client_options.client_cert_source, self._use_client_cert
        )
        self._universe_domain = SubscriptionsServiceClient._get_universe_domain(
            universe_domain_opt, self._universe_domain_env
        )
        self._api_endpoint = None  # updated below, depending on `transport`

        # Initialize the universe domain validation.
        self._is_universe_domain_valid = False

        api_key_value = getattr(self._client_options, "api_key", None)
        if api_key_value and credentials:
            raise ValueError(
                "client_options.api_key and credentials are mutually exclusive"
            )

        # Save or instantiate the transport.
        # Ordinarily, we provide the transport, but allowing a custom transport
        # instance provides an extensibility point for unusual situations.
        transport_provided = isinstance(transport, SubscriptionsServiceTransport)
        if transport_provided:
            # transport is a SubscriptionsServiceTransport instance.
            if credentials or self._client_options.credentials_file or api_key_value:
                raise ValueError(
                    "When providing a transport instance, "
                    "provide its credentials directly."
                )
            if self._client_options.scopes:
                raise ValueError(
                    "When providing a transport instance, provide its scopes "
                    "directly."
                )
            self._transport = cast(SubscriptionsServiceTransport, transport)
            self._api_endpoint = self._transport.host

        self._api_endpoint = (
            self._api_endpoint
            or SubscriptionsServiceClient._get_api_endpoint(
                self._client_options.api_endpoint,
                self._client_cert_source,
                self._universe_domain,
                self._use_mtls_endpoint,
            )
        )

        if not transport_provided:
            import google.auth._default  # type: ignore

            if api_key_value and hasattr(
                google.auth._default, "get_api_key_credentials"
            ):
                credentials = google.auth._default.get_api_key_credentials(
                    api_key_value
                )

            Transport = type(self).get_transport_class(cast(str, transport))
            self._transport = Transport(
                credentials=credentials,
                credentials_file=self._client_options.credentials_file,
                host=self._api_endpoint,
                scopes=self._client_options.scopes,
                client_cert_source_for_mtls=self._client_cert_source,
                quota_project_id=self._client_options.quota_project_id,
                client_info=client_info,
                always_use_jwt_access=True,
                api_audience=self._client_options.api_audience,
            )

    def create_subscription(
        self,
        request: Optional[
            Union[subscriptions_service.CreateSubscriptionRequest, dict]
        ] = None,
        *,
        subscription: Optional[subscription_resource.Subscription] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation.Operation:
        r"""Creates a Google Workspace subscription. To learn how to use
        this method, see `Create a Google Workspace
        subscription <https://developers.google.com/workspace/events/guides/create-subscription>`__.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.apps import events_subscriptions_v1

            def sample_create_subscription():
                # Create a client
                client = events_subscriptions_v1.SubscriptionsServiceClient()

                # Initialize request argument(s)
                subscription = events_subscriptions_v1.Subscription()
                subscription.target_resource = "target_resource_value"
                subscription.event_types = ['event_types_value1', 'event_types_value2']
                subscription.notification_endpoint.pubsub_topic = "pubsub_topic_value"

                request = events_subscriptions_v1.CreateSubscriptionRequest(
                    subscription=subscription,
                )

                # Make the request
                operation = client.create_subscription(request=request)

                print("Waiting for operation to complete...")

                response = operation.result()

                # Handle the response
                print(response)

        Args:
            request (Union[google.apps.events_subscriptions_v1.types.CreateSubscriptionRequest, dict]):
                The request object. The request message for
                [SubscriptionsService.CreateSubscription][google.apps.events.subscriptions.v1.SubscriptionsService.CreateSubscription].
            subscription (google.apps.events_subscriptions_v1.types.Subscription):
                Required. The subscription resource
                to create.

                This corresponds to the ``subscription`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation.Operation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.apps.events_subscriptions_v1.types.Subscription` A subscription to receive events about a Google Workspace resource. To learn
                   more about subscriptions, see the [Google Workspace
                   Events API
                   overview](\ https://developers.google.com/workspace/events).

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([subscription])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # Minor optimization to avoid making a copy if the user passes
        # in a subscriptions_service.CreateSubscriptionRequest.
        # There's no risk of modifying the input as we've already verified
        # there are no flattened fields.
        if not isinstance(request, subscriptions_service.CreateSubscriptionRequest):
            request = subscriptions_service.CreateSubscriptionRequest(request)
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if subscription is not None:
                request.subscription = subscription

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[self._transport.create_subscription]

        # Validate the universe domain.
        self._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation.from_gapic(
            response,
            self._transport.operations_client,
            subscription_resource.Subscription,
            metadata_type=subscriptions_service.CreateSubscriptionMetadata,
        )

        # Done; return the response.
        return response

    def delete_subscription(
        self,
        request: Optional[
            Union[subscriptions_service.DeleteSubscriptionRequest, dict]
        ] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation.Operation:
        r"""Deletes a Google Workspace subscription. To learn how to use
        this method, see `Delete a Google Workspace
        subscription <https://developers.google.com/workspace/events/guides/delete-subscription>`__.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.apps import events_subscriptions_v1

            def sample_delete_subscription():
                # Create a client
                client = events_subscriptions_v1.SubscriptionsServiceClient()

                # Initialize request argument(s)
                request = events_subscriptions_v1.DeleteSubscriptionRequest(
                    name="name_value",
                )

                # Make the request
                operation = client.delete_subscription(request=request)

                print("Waiting for operation to complete...")

                response = operation.result()

                # Handle the response
                print(response)

        Args:
            request (Union[google.apps.events_subscriptions_v1.types.DeleteSubscriptionRequest, dict]):
                The request object. The request message for
                [SubscriptionsService.DeleteSubscription][google.apps.events.subscriptions.v1.SubscriptionsService.DeleteSubscription].
            name (str):
                Required. Resource name of the subscription to delete.

                Format: ``subscriptions/{subscription}``

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation.Operation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.protobuf.empty_pb2.Empty` A generic empty message that you can re-use to avoid defining duplicated
                   empty messages in your APIs. A typical example is to
                   use it as the request or the response type of an API
                   method. For instance:

                      service Foo {
                         rpc Bar(google.protobuf.Empty) returns
                         (google.protobuf.Empty);

                      }

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # Minor optimization to avoid making a copy if the user passes
        # in a subscriptions_service.DeleteSubscriptionRequest.
        # There's no risk of modifying the input as we've already verified
        # there are no flattened fields.
        if not isinstance(request, subscriptions_service.DeleteSubscriptionRequest):
            request = subscriptions_service.DeleteSubscriptionRequest(request)
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if name is not None:
                request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[self._transport.delete_subscription]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation.from_gapic(
            response,
            self._transport.operations_client,
            empty_pb2.Empty,
            metadata_type=subscriptions_service.DeleteSubscriptionMetadata,
        )

        # Done; return the response.
        return response

    def get_subscription(
        self,
        request: Optional[
            Union[subscriptions_service.GetSubscriptionRequest, dict]
        ] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> subscription_resource.Subscription:
        r"""Gets details about a Google Workspace subscription. To learn how
        to use this method, see `Get details about a Google Workspace
        subscription <https://developers.google.com/workspace/events/guides/get-subscription>`__.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.apps import events_subscriptions_v1

            def sample_get_subscription():
                # Create a client
                client = events_subscriptions_v1.SubscriptionsServiceClient()

                # Initialize request argument(s)
                request = events_subscriptions_v1.GetSubscriptionRequest(
                    name="name_value",
                )

                # Make the request
                response = client.get_subscription(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.apps.events_subscriptions_v1.types.GetSubscriptionRequest, dict]):
                The request object. The request message for
                [SubscriptionsService.GetSubscription][google.apps.events.subscriptions.v1.SubscriptionsService.GetSubscription].
            name (str):
                Required. Resource name of the subscription.

                Format: ``subscriptions/{subscription}``

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.apps.events_subscriptions_v1.types.Subscription:
                A subscription to receive events about a Google Workspace resource. To learn
                   more about subscriptions, see the [Google Workspace
                   Events API
                   overview](\ https://developers.google.com/workspace/events).

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # Minor optimization to avoid making a copy if the user passes
        # in a subscriptions_service.GetSubscriptionRequest.
        # There's no risk of modifying the input as we've already verified
        # there are no flattened fields.
        if not isinstance(request, subscriptions_service.GetSubscriptionRequest):
            request = subscriptions_service.GetSubscriptionRequest(request)
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if name is not None:
                request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[self._transport.get_subscription]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    def list_subscriptions(
        self,
        request: Optional[
            Union[subscriptions_service.ListSubscriptionsRequest, dict]
        ] = None,
        *,
        filter: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListSubscriptionsPager:
        r"""Lists Google Workspace subscriptions. To learn how to use this
        method, see `List Google Workspace
        subscriptions <https://developers.google.com/workspace/events/guides/list-subscriptions>`__.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.apps import events_subscriptions_v1

            def sample_list_subscriptions():
                # Create a client
                client = events_subscriptions_v1.SubscriptionsServiceClient()

                # Initialize request argument(s)
                request = events_subscriptions_v1.ListSubscriptionsRequest(
                    filter="filter_value",
                )

                # Make the request
                page_result = client.list_subscriptions(request=request)

                # Handle the response
                for response in page_result:
                    print(response)

        Args:
            request (Union[google.apps.events_subscriptions_v1.types.ListSubscriptionsRequest, dict]):
                The request object. The request message for
                [SubscriptionsService.ListSubscriptions][google.apps.events.subscriptions.v1.SubscriptionsService.ListSubscriptions].
            filter (str):
                Required. A query filter.

                You can filter subscriptions by event type
                (``event_types``) and target resource
                (``target_resource``).

                You must specify at least one event type in your query.
                To filter for multiple event types, use the ``OR``
                operator.

                To filter by both event type and target resource, use
                the ``AND`` operator and specify the full resource name,
                such as ``//chat.googleapis.com/spaces/{space}``.

                For example, the following queries are valid:

                ::

                   event_types:"google.workspace.chat.membership.v1.updated" OR
                     event_types:"google.workspace.chat.message.v1.created"

                   event_types:"google.workspace.chat.message.v1.created" AND
                     target_resource="//chat.googleapis.com/spaces/{space}"

                   ( event_types:"google.workspace.chat.membership.v1.updated" OR
                     event_types:"google.workspace.chat.message.v1.created" ) AND
                     target_resource="//chat.googleapis.com/spaces/{space}"

                The server rejects invalid queries with an
                ``INVALID_ARGUMENT`` error.

                This corresponds to the ``filter`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.apps.events_subscriptions_v1.services.subscriptions_service.pagers.ListSubscriptionsPager:
                The response message for
                   [SubscriptionsService.ListSubscriptions][google.apps.events.subscriptions.v1.SubscriptionsService.ListSubscriptions].

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([filter])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # Minor optimization to avoid making a copy if the user passes
        # in a subscriptions_service.ListSubscriptionsRequest.
        # There's no risk of modifying the input as we've already verified
        # there are no flattened fields.
        if not isinstance(request, subscriptions_service.ListSubscriptionsRequest):
            request = subscriptions_service.ListSubscriptionsRequest(request)
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if filter is not None:
                request.filter = filter

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[self._transport.list_subscriptions]

        # Validate the universe domain.
        self._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__iter__` convenience method.
        response = pagers.ListSubscriptionsPager(
            method=rpc,
            request=request,
            response=response,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    def update_subscription(
        self,
        request: Optional[
            Union[subscriptions_service.UpdateSubscriptionRequest, dict]
        ] = None,
        *,
        subscription: Optional[subscription_resource.Subscription] = None,
        update_mask: Optional[field_mask_pb2.FieldMask] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation.Operation:
        r"""Updates or renews a Google Workspace subscription. To learn how
        to use this method, see `Update or renew a Google Workspace
        subscription <https://developers.google.com/workspace/events/guides/update-subscription>`__.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.apps import events_subscriptions_v1

            def sample_update_subscription():
                # Create a client
                client = events_subscriptions_v1.SubscriptionsServiceClient()

                # Initialize request argument(s)
                subscription = events_subscriptions_v1.Subscription()
                subscription.target_resource = "target_resource_value"
                subscription.event_types = ['event_types_value1', 'event_types_value2']
                subscription.notification_endpoint.pubsub_topic = "pubsub_topic_value"

                request = events_subscriptions_v1.UpdateSubscriptionRequest(
                    subscription=subscription,
                )

                # Make the request
                operation = client.update_subscription(request=request)

                print("Waiting for operation to complete...")

                response = operation.result()

                # Handle the response
                print(response)

        Args:
            request (Union[google.apps.events_subscriptions_v1.types.UpdateSubscriptionRequest, dict]):
                The request object. The request message for
                [SubscriptionsService.UpdateSubscription][google.apps.events.subscriptions.v1.SubscriptionsService.UpdateSubscription].
            subscription (google.apps.events_subscriptions_v1.types.Subscription):
                Required. The subscription to update.

                The subscription's ``name`` field is used to identify
                the subscription to update.

                This corresponds to the ``subscription`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            update_mask (google.protobuf.field_mask_pb2.FieldMask):
                Optional. Required. The field to update.

                You can update one of the following fields in a
                subscription:

                -  [``expire_time``][google.apps.events.subscriptions.v1.Subscription.expire_time]:
                   The timestamp when the subscription expires.
                -  [``ttl``][google.apps.events.subscriptions.v1.Subscription.ttl]:
                   The time-to-live (TTL) or duration of the
                   subscription.

                This corresponds to the ``update_mask`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation.Operation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.apps.events_subscriptions_v1.types.Subscription` A subscription to receive events about a Google Workspace resource. To learn
                   more about subscriptions, see the [Google Workspace
                   Events API
                   overview](\ https://developers.google.com/workspace/events).

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([subscription, update_mask])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # Minor optimization to avoid making a copy if the user passes
        # in a subscriptions_service.UpdateSubscriptionRequest.
        # There's no risk of modifying the input as we've already verified
        # there are no flattened fields.
        if not isinstance(request, subscriptions_service.UpdateSubscriptionRequest):
            request = subscriptions_service.UpdateSubscriptionRequest(request)
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if subscription is not None:
                request.subscription = subscription
            if update_mask is not None:
                request.update_mask = update_mask

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[self._transport.update_subscription]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("subscription.name", request.subscription.name),)
            ),
        )

        # Validate the universe domain.
        self._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation.from_gapic(
            response,
            self._transport.operations_client,
            subscription_resource.Subscription,
            metadata_type=subscriptions_service.UpdateSubscriptionMetadata,
        )

        # Done; return the response.
        return response

    def reactivate_subscription(
        self,
        request: Optional[
            Union[subscriptions_service.ReactivateSubscriptionRequest, dict]
        ] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation.Operation:
        r"""Reactivates a suspended Google Workspace subscription.

        This method resets your subscription's ``State`` field to
        ``ACTIVE``. Before you use this method, you must fix the error
        that suspended the subscription. To learn how to use this
        method, see `Reactivate a Google Workspace
        subscription <https://developers.google.com/workspace/events/guides/reactivate-subscription>`__.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.apps import events_subscriptions_v1

            def sample_reactivate_subscription():
                # Create a client
                client = events_subscriptions_v1.SubscriptionsServiceClient()

                # Initialize request argument(s)
                request = events_subscriptions_v1.ReactivateSubscriptionRequest(
                    name="name_value",
                )

                # Make the request
                operation = client.reactivate_subscription(request=request)

                print("Waiting for operation to complete...")

                response = operation.result()

                # Handle the response
                print(response)

        Args:
            request (Union[google.apps.events_subscriptions_v1.types.ReactivateSubscriptionRequest, dict]):
                The request object. The request message for
                [SubscriptionsService.ReactivateSubscription][google.apps.events.subscriptions.v1.SubscriptionsService.ReactivateSubscription].
            name (str):
                Required. Resource name of the subscription.

                Format: ``subscriptions/{subscription}``

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation.Operation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.apps.events_subscriptions_v1.types.Subscription` A subscription to receive events about a Google Workspace resource. To learn
                   more about subscriptions, see the [Google Workspace
                   Events API
                   overview](\ https://developers.google.com/workspace/events).

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # Minor optimization to avoid making a copy if the user passes
        # in a subscriptions_service.ReactivateSubscriptionRequest.
        # There's no risk of modifying the input as we've already verified
        # there are no flattened fields.
        if not isinstance(request, subscriptions_service.ReactivateSubscriptionRequest):
            request = subscriptions_service.ReactivateSubscriptionRequest(request)
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if name is not None:
                request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[self._transport.reactivate_subscription]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation.from_gapic(
            response,
            self._transport.operations_client,
            subscription_resource.Subscription,
            metadata_type=subscriptions_service.ReactivateSubscriptionMetadata,
        )

        # Done; return the response.
        return response

    def __enter__(self) -> "SubscriptionsServiceClient":
        return self

    def __exit__(self, type, value, traceback):
        """Releases underlying transport's resources.

        .. warning::
            ONLY use as a context manager if the transport is NOT shared
            with other clients! Exiting the with block will CLOSE the transport
            and may cause errors in other clients!
        """
        self.transport.close()

    def get_operation(
        self,
        request: Optional[operations_pb2.GetOperationRequest] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operations_pb2.Operation:
        r"""Gets the latest state of a long-running operation.

        Args:
            request (:class:`~.operations_pb2.GetOperationRequest`):
                The request object. Request message for
                `GetOperation` method.
            retry (google.api_core.retry.Retry): Designation of what errors,
                    if any, should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        Returns:
            ~.operations_pb2.Operation:
                An ``Operation`` object.
        """
        # Create or coerce a protobuf request object.
        # The request isn't a proto-plus wrapped type,
        # so it must be constructed via keyword expansion.
        if isinstance(request, dict):
            request = operations_pb2.GetOperationRequest(**request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method.wrap_method(
            self._transport.get_operation,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)


__all__ = ("SubscriptionsServiceClient",)



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import (
    Dict,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
)

from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.api_core.client_options import ClientOptions
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.apps.events_subscriptions_v1 import gapic_version as package_version

try:
    OptionalRetry = Union[retries.AsyncRetry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object, None]  # type: ignore

from google.api_core import operation  # type: ignore
from google.api_core import operation_async  # type: ignore
from google.longrunning import operations_pb2  # type: ignore
from google.protobuf import duration_pb2  # type: ignore
from google.protobuf import empty_pb2  # type: ignore
from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore

from google.apps.events_subscriptions_v1.services.subscriptions_service import pagers
from google.apps.events_subscriptions_v1.types import (
    subscription_resource,
    subscriptions_service,
)

from .client import SubscriptionsServiceClient
from .transports.base import DEFAULT_CLIENT_INFO, SubscriptionsServiceTransport
from .transports.grpc_asyncio import SubscriptionsServiceGrpcAsyncIOTransport


class SubscriptionsServiceAsyncClient:
    """A service that manages subscriptions to Google Workspace
    events.
    """

    _client: SubscriptionsServiceClient

    # Copy defaults from the synchronous client for use here.
    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = SubscriptionsServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = SubscriptionsServiceClient.DEFAULT_MTLS_ENDPOINT
    _DEFAULT_ENDPOINT_TEMPLATE = SubscriptionsServiceClient._DEFAULT_ENDPOINT_TEMPLATE
    _DEFAULT_UNIVERSE = SubscriptionsServiceClient._DEFAULT_UNIVERSE

    subscription_path = staticmethod(SubscriptionsServiceClient.subscription_path)
    parse_subscription_path = staticmethod(
        SubscriptionsServiceClient.parse_subscription_path
    )
    topic_path = staticmethod(SubscriptionsServiceClient.topic_path)
    parse_topic_path = staticmethod(SubscriptionsServiceClient.parse_topic_path)
    user_path = staticmethod(SubscriptionsServiceClient.user_path)
    parse_user_path = staticmethod(SubscriptionsServiceClient.parse_user_path)
    common_billing_account_path = staticmethod(
        SubscriptionsServiceClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        SubscriptionsServiceClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(SubscriptionsServiceClient.common_folder_path)
    parse_common_folder_path = staticmethod(
        SubscriptionsServiceClient.parse_common_folder_path
    )
    common_organization_path = staticmethod(
        SubscriptionsServiceClient.common_organization_path
    )
    parse_common_organization_path = staticmethod(
        SubscriptionsServiceClient.parse_common_organization_path
    )
    common_project_path = staticmethod(SubscriptionsServiceClient.common_project_path)
    parse_common_project_path = staticmethod(
        SubscriptionsServiceClient.parse_common_project_path
    )
    common_location_path = staticmethod(SubscriptionsServiceClient.common_location_path)
    parse_common_location_path = staticmethod(
        SubscriptionsServiceClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            SubscriptionsServiceAsyncClient: The constructed client.
        """
        return SubscriptionsServiceClient.from_service_account_info.__func__(SubscriptionsServiceAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            SubscriptionsServiceAsyncClient: The constructed client.
        """
        return SubscriptionsServiceClient.from_service_account_file.__func__(SubscriptionsServiceAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return SubscriptionsServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> SubscriptionsServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            SubscriptionsServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    @property
    def api_endpoint(self):
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._client._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used
                by the client instance.
        """
        return self._client._universe_domain

    get_transport_class = functools.partial(
        type(SubscriptionsServiceClient).get_transport_class,
        type(SubscriptionsServiceClient),
    )

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Union[str, SubscriptionsServiceTransport] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the subscriptions service async client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.SubscriptionsServiceTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = SubscriptionsServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def create_subscription(
        self,
        request: Optional[
            Union[subscriptions_service.CreateSubscriptionRequest, dict]
        ] = None,
        *,
        subscription: Optional[subscription_resource.Subscription] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Creates a Google Workspace subscription. To learn how to use
        this method, see `Create a Google Workspace
        subscription <https://developers.google.com/workspace/events/guides/create-subscription>`__.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.apps import events_subscriptions_v1

            async def sample_create_subscription():
                # Create a client
                client = events_subscriptions_v1.SubscriptionsServiceAsyncClient()

                # Initialize request argument(s)
                subscription = events_subscriptions_v1.Subscription()
                subscription.target_resource = "target_resource_value"
                subscription.event_types = ['event_types_value1', 'event_types_value2']
                subscription.notification_endpoint.pubsub_topic = "pubsub_topic_value"

                request = events_subscriptions_v1.CreateSubscriptionRequest(
                    subscription=subscription,
                )

                # Make the request
                operation = client.create_subscription(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.apps.events_subscriptions_v1.types.CreateSubscriptionRequest, dict]]):
                The request object. The request message for
                [SubscriptionsService.CreateSubscription][google.apps.events.subscriptions.v1.SubscriptionsService.CreateSubscription].
            subscription (:class:`google.apps.events_subscriptions_v1.types.Subscription`):
                Required. The subscription resource
                to create.

                This corresponds to the ``subscription`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.apps.events_subscriptions_v1.types.Subscription` A subscription to receive events about a Google Workspace resource. To learn
                   more about subscriptions, see the [Google Workspace
                   Events API
                   overview](\ https://developers.google.com/workspace/events).

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([subscription])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = subscriptions_service.CreateSubscriptionRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if subscription is not None:
            request.subscription = subscription

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.create_subscription,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            subscription_resource.Subscription,
            metadata_type=subscriptions_service.CreateSubscriptionMetadata,
        )

        # Done; return the response.
        return response

    async def delete_subscription(
        self,
        request: Optional[
            Union[subscriptions_service.DeleteSubscriptionRequest, dict]
        ] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Deletes a Google Workspace subscription. To learn how to use
        this method, see `Delete a Google Workspace
        subscription <https://developers.google.com/workspace/events/guides/delete-subscription>`__.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.apps import events_subscriptions_v1

            async def sample_delete_subscription():
                # Create a client
                client = events_subscriptions_v1.SubscriptionsServiceAsyncClient()

                # Initialize request argument(s)
                request = events_subscriptions_v1.DeleteSubscriptionRequest(
                    name="name_value",
                )

                # Make the request
                operation = client.delete_subscription(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.apps.events_subscriptions_v1.types.DeleteSubscriptionRequest, dict]]):
                The request object. The request message for
                [SubscriptionsService.DeleteSubscription][google.apps.events.subscriptions.v1.SubscriptionsService.DeleteSubscription].
            name (:class:`str`):
                Required. Resource name of the subscription to delete.

                Format: ``subscriptions/{subscription}``

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.protobuf.empty_pb2.Empty` A generic empty message that you can re-use to avoid defining duplicated
                   empty messages in your APIs. A typical example is to
                   use it as the request or the response type of an API
                   method. For instance:

                      service Foo {
                         rpc Bar(google.protobuf.Empty) returns
                         (google.protobuf.Empty);

                      }

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = subscriptions_service.DeleteSubscriptionRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.delete_subscription,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            empty_pb2.Empty,
            metadata_type=subscriptions_service.DeleteSubscriptionMetadata,
        )

        # Done; return the response.
        return response

    async def get_subscription(
        self,
        request: Optional[
            Union[subscriptions_service.GetSubscriptionRequest, dict]
        ] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> subscription_resource.Subscription:
        r"""Gets details about a Google Workspace subscription. To learn how
        to use this method, see `Get details about a Google Workspace
        subscription <https://developers.google.com/workspace/events/guides/get-subscription>`__.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.apps import events_subscriptions_v1

            async def sample_get_subscription():
                # Create a client
                client = events_subscriptions_v1.SubscriptionsServiceAsyncClient()

                # Initialize request argument(s)
                request = events_subscriptions_v1.GetSubscriptionRequest(
                    name="name_value",
                )

                # Make the request
                response = await client.get_subscription(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.apps.events_subscriptions_v1.types.GetSubscriptionRequest, dict]]):
                The request object. The request message for
                [SubscriptionsService.GetSubscription][google.apps.events.subscriptions.v1.SubscriptionsService.GetSubscription].
            name (:class:`str`):
                Required. Resource name of the subscription.

                Format: ``subscriptions/{subscription}``

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.apps.events_subscriptions_v1.types.Subscription:
                A subscription to receive events about a Google Workspace resource. To learn
                   more about subscriptions, see the [Google Workspace
                   Events API
                   overview](\ https://developers.google.com/workspace/events).

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = subscriptions_service.GetSubscriptionRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_subscription,
            default_retry=retries.AsyncRetry(
                initial=1.0,
                maximum=10.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=60.0,
            ),
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_subscriptions(
        self,
        request: Optional[
            Union[subscriptions_service.ListSubscriptionsRequest, dict]
        ] = None,
        *,
        filter: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListSubscriptionsAsyncPager:
        r"""Lists Google Workspace subscriptions. To learn how to use this
        method, see `List Google Workspace
        subscriptions <https://developers.google.com/workspace/events/guides/list-subscriptions>`__.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.apps import events_subscriptions_v1

            async def sample_list_subscriptions():
                # Create a client
                client = events_subscriptions_v1.SubscriptionsServiceAsyncClient()

                # Initialize request argument(s)
                request = events_subscriptions_v1.ListSubscriptionsRequest(
                    filter="filter_value",
                )

                # Make the request
                page_result = client.list_subscriptions(request=request)

                # Handle the response
                async for response in page_result:
                    print(response)

        Args:
            request (Optional[Union[google.apps.events_subscriptions_v1.types.ListSubscriptionsRequest, dict]]):
                The request object. The request message for
                [SubscriptionsService.ListSubscriptions][google.apps.events.subscriptions.v1.SubscriptionsService.ListSubscriptions].
            filter (:class:`str`):
                Required. A query filter.

                You can filter subscriptions by event type
                (``event_types``) and target resource
                (``target_resource``).

                You must specify at least one event type in your query.
                To filter for multiple event types, use the ``OR``
                operator.

                To filter by both event type and target resource, use
                the ``AND`` operator and specify the full resource name,
                such as ``//chat.googleapis.com/spaces/{space}``.

                For example, the following queries are valid:

                ::

                   event_types:"google.workspace.chat.membership.v1.updated" OR
                     event_types:"google.workspace.chat.message.v1.created"

                   event_types:"google.workspace.chat.message.v1.created" AND
                     target_resource="//chat.googleapis.com/spaces/{space}"

                   ( event_types:"google.workspace.chat.membership.v1.updated" OR
                     event_types:"google.workspace.chat.message.v1.created" ) AND
                     target_resource="//chat.googleapis.com/spaces/{space}"

                The server rejects invalid queries with an
                ``INVALID_ARGUMENT`` error.

                This corresponds to the ``filter`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.apps.events_subscriptions_v1.services.subscriptions_service.pagers.ListSubscriptionsAsyncPager:
                The response message for
                   [SubscriptionsService.ListSubscriptions][google.apps.events.subscriptions.v1.SubscriptionsService.ListSubscriptions].

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([filter])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = subscriptions_service.ListSubscriptionsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if filter is not None:
            request.filter = filter

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_subscriptions,
            default_retry=retries.AsyncRetry(
                initial=1.0,
                maximum=10.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=60.0,
            ),
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListSubscriptionsAsyncPager(
            method=rpc,
            request=request,
            response=response,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def update_subscription(
        self,
        request: Optional[
            Union[subscriptions_service.UpdateSubscriptionRequest, dict]
        ] = None,
        *,
        subscription: Optional[subscription_resource.Subscription] = None,
        update_mask: Optional[field_mask_pb2.FieldMask] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Updates or renews a Google Workspace subscription. To learn how
        to use this method, see `Update or renew a Google Workspace
        subscription <https://developers.google.com/workspace/events/guides/update-subscription>`__.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.apps import events_subscriptions_v1

            async def sample_update_subscription():
                # Create a client
                client = events_subscriptions_v1.SubscriptionsServiceAsyncClient()

                # Initialize request argument(s)
                subscription = events_subscriptions_v1.Subscription()
                subscription.target_resource = "target_resource_value"
                subscription.event_types = ['event_types_value1', 'event_types_value2']
                subscription.notification_endpoint.pubsub_topic = "pubsub_topic_value"

                request = events_subscriptions_v1.UpdateSubscriptionRequest(
                    subscription=subscription,
                )

                # Make the request
                operation = client.update_subscription(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.apps.events_subscriptions_v1.types.UpdateSubscriptionRequest, dict]]):
                The request object. The request message for
                [SubscriptionsService.UpdateSubscription][google.apps.events.subscriptions.v1.SubscriptionsService.UpdateSubscription].
            subscription (:class:`google.apps.events_subscriptions_v1.types.Subscription`):
                Required. The subscription to update.

                The subscription's ``name`` field is used to identify
                the subscription to update.

                This corresponds to the ``subscription`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            update_mask (:class:`google.protobuf.field_mask_pb2.FieldMask`):
                Optional. Required. The field to update.

                You can update one of the following fields in a
                subscription:

                -  [``expire_time``][google.apps.events.subscriptions.v1.Subscription.expire_time]:
                   The timestamp when the subscription expires.
                -  [``ttl``][google.apps.events.subscriptions.v1.Subscription.ttl]:
                   The time-to-live (TTL) or duration of the
                   subscription.

                This corresponds to the ``update_mask`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.apps.events_subscriptions_v1.types.Subscription` A subscription to receive events about a Google Workspace resource. To learn
                   more about subscriptions, see the [Google Workspace
                   Events API
                   overview](\ https://developers.google.com/workspace/events).

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([subscription, update_mask])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = subscriptions_service.UpdateSubscriptionRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if subscription is not None:
            request.subscription = subscription
        if update_mask is not None:
            request.update_mask = update_mask

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.update_subscription,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("subscription.name", request.subscription.name),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            subscription_resource.Subscription,
            metadata_type=subscriptions_service.UpdateSubscriptionMetadata,
        )

        # Done; return the response.
        return response

    async def reactivate_subscription(
        self,
        request: Optional[
            Union[subscriptions_service.ReactivateSubscriptionRequest, dict]
        ] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Reactivates a suspended Google Workspace subscription.

        This method resets your subscription's ``State`` field to
        ``ACTIVE``. Before you use this method, you must fix the error
        that suspended the subscription. To learn how to use this
        method, see `Reactivate a Google Workspace
        subscription <https://developers.google.com/workspace/events/guides/reactivate-subscription>`__.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.apps import events_subscriptions_v1

            async def sample_reactivate_subscription():
                # Create a client
                client = events_subscriptions_v1.SubscriptionsServiceAsyncClient()

                # Initialize request argument(s)
                request = events_subscriptions_v1.ReactivateSubscriptionRequest(
                    name="name_value",
                )

                # Make the request
                operation = client.reactivate_subscription(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.apps.events_subscriptions_v1.types.ReactivateSubscriptionRequest, dict]]):
                The request object. The request message for
                [SubscriptionsService.ReactivateSubscription][google.apps.events.subscriptions.v1.SubscriptionsService.ReactivateSubscription].
            name (:class:`str`):
                Required. Resource name of the subscription.

                Format: ``subscriptions/{subscription}``

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.apps.events_subscriptions_v1.types.Subscription` A subscription to receive events about a Google Workspace resource. To learn
                   more about subscriptions, see the [Google Workspace
                   Events API
                   overview](\ https://developers.google.com/workspace/events).

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = subscriptions_service.ReactivateSubscriptionRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.reactivate_subscription,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            subscription_resource.Subscription,
            metadata_type=subscriptions_service.ReactivateSubscriptionMetadata,
        )

        # Done; return the response.
        return response

    async def get_operation(
        self,
        request: Optional[operations_pb2.GetOperationRequest] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operations_pb2.Operation:
        r"""Gets the latest state of a long-running operation.

        Args:
            request (:class:`~.operations_pb2.GetOperationRequest`):
                The request object. Request message for
                `GetOperation` method.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors,
                    if any, should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        Returns:
            ~.operations_pb2.Operation:
                An ``Operation`` object.
        """
        # Create or coerce a protobuf request object.
        # The request isn't a proto-plus wrapped type,
        # so it must be constructed via keyword expansion.
        if isinstance(request, dict):
            request = operations_pb2.GetOperationRequest(**request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_operation,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self) -> "SubscriptionsServiceAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)


__all__ = ("SubscriptionsServiceAsyncClient",)



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import field_mask_pb2  # type: ignore
import proto  # type: ignore

from google.apps.events_subscriptions_v1.types import subscription_resource

__protobuf__ = proto.module(
    package="google.apps.events.subscriptions.v1",
    manifest={
        "CreateSubscriptionRequest",
        "DeleteSubscriptionRequest",
        "GetSubscriptionRequest",
        "UpdateSubscriptionRequest",
        "ReactivateSubscriptionRequest",
        "ListSubscriptionsRequest",
        "ListSubscriptionsResponse",
        "UpdateSubscriptionMetadata",
        "CreateSubscriptionMetadata",
        "DeleteSubscriptionMetadata",
        "ReactivateSubscriptionMetadata",
    },
)


class CreateSubscriptionRequest(proto.Message):
    r"""The request message for
    [SubscriptionsService.CreateSubscription][google.apps.events.subscriptions.v1.SubscriptionsService.CreateSubscription].

    Attributes:
        subscription (google.apps.events_subscriptions_v1.types.Subscription):
            Required. The subscription resource to
            create.
        validate_only (bool):
            Optional. If set to ``true``, validates and previews the
            request, but doesn't create the subscription.
    """

    subscription: subscription_resource.Subscription = proto.Field(
        proto.MESSAGE,
        number=1,
        message=subscription_resource.Subscription,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class DeleteSubscriptionRequest(proto.Message):
    r"""The request message for
    [SubscriptionsService.DeleteSubscription][google.apps.events.subscriptions.v1.SubscriptionsService.DeleteSubscription].

    Attributes:
        name (str):
            Required. Resource name of the subscription to delete.

            Format: ``subscriptions/{subscription}``
        validate_only (bool):
            Optional. If set to ``true``, validates and previews the
            request, but doesn't delete the subscription.
        allow_missing (bool):
            Optional. If set to ``true`` and the subscription isn't
            found, the request succeeds but doesn't delete the
            subscription.
        etag (str):
            Optional. Etag of the subscription.

            If present, it must match with the server's etag. Otherwise,
            request fails with the status ``ABORTED``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=2,
    )
    allow_missing: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    etag: str = proto.Field(
        proto.STRING,
        number=4,
    )


class GetSubscriptionRequest(proto.Message):
    r"""The request message for
    [SubscriptionsService.GetSubscription][google.apps.events.subscriptions.v1.SubscriptionsService.GetSubscription].

    Attributes:
        name (str):
            Required. Resource name of the subscription.

            Format: ``subscriptions/{subscription}``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class UpdateSubscriptionRequest(proto.Message):
    r"""The request message for
    [SubscriptionsService.UpdateSubscription][google.apps.events.subscriptions.v1.SubscriptionsService.UpdateSubscription].

    Attributes:
        subscription (google.apps.events_subscriptions_v1.types.Subscription):
            Required. The subscription to update.

            The subscription's ``name`` field is used to identify the
            subscription to update.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Optional. Required. The field to update.

            You can update one of the following fields in a
            subscription:

            -  [``expire_time``][google.apps.events.subscriptions.v1.Subscription.expire_time]:
               The timestamp when the subscription expires.
            -  [``ttl``][google.apps.events.subscriptions.v1.Subscription.ttl]:
               The time-to-live (TTL) or duration of the subscription.
        validate_only (bool):
            Optional. If set to ``true``, validates and previews the
            request, but doesn't update the subscription.
    """

    subscription: subscription_resource.Subscription = proto.Field(
        proto.MESSAGE,
        number=1,
        message=subscription_resource.Subscription,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class ReactivateSubscriptionRequest(proto.Message):
    r"""The request message for
    [SubscriptionsService.ReactivateSubscription][google.apps.events.subscriptions.v1.SubscriptionsService.ReactivateSubscription].

    Attributes:
        name (str):
            Required. Resource name of the subscription.

            Format: ``subscriptions/{subscription}``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListSubscriptionsRequest(proto.Message):
    r"""The request message for
    [SubscriptionsService.ListSubscriptions][google.apps.events.subscriptions.v1.SubscriptionsService.ListSubscriptions].

    Attributes:
        page_size (int):
            Optional. The maximum number of subscriptions to return. The
            service might return fewer than this value.

            If unspecified or set to ``0``, up to 50 subscriptions are
            returned.

            The maximum value is 100. If you specify a value more than
            100, the system only returns 100 subscriptions.
        page_token (str):
            Optional. A page token, received from a
            previous list subscriptions call. Provide this
            parameter to retrieve the subsequent page.

            When paginating, the filter value should match
            the call that provided the page token. Passing a
            different value might lead to unexpected
            results.
        filter (str):
            Required. A query filter.

            You can filter subscriptions by event type (``event_types``)
            and target resource (``target_resource``).

            You must specify at least one event type in your query. To
            filter for multiple event types, use the ``OR`` operator.

            To filter by both event type and target resource, use the
            ``AND`` operator and specify the full resource name, such as
            ``//chat.googleapis.com/spaces/{space}``.

            For example, the following queries are valid:

            ::

               event_types:"google.workspace.chat.membership.v1.updated" OR
                 event_types:"google.workspace.chat.message.v1.created"

               event_types:"google.workspace.chat.message.v1.created" AND
                 target_resource="//chat.googleapis.com/spaces/{space}"

               ( event_types:"google.workspace.chat.membership.v1.updated" OR
                 event_types:"google.workspace.chat.message.v1.created" ) AND
                 target_resource="//chat.googleapis.com/spaces/{space}"

            The server rejects invalid queries with an
            ``INVALID_ARGUMENT`` error.
    """

    page_size: int = proto.Field(
        proto.INT32,
        number=1,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListSubscriptionsResponse(proto.Message):
    r"""The response message for
    [SubscriptionsService.ListSubscriptions][google.apps.events.subscriptions.v1.SubscriptionsService.ListSubscriptions].

    Attributes:
        subscriptions (MutableSequence[google.apps.events_subscriptions_v1.types.Subscription]):
            List of subscriptions.
        next_page_token (str):
            A token, which can be sent as ``page_token`` to retrieve the
            next page. If this field is omitted, there are no subsequent
            pages.
    """

    @property
    def raw_page(self):
        return self

    subscriptions: MutableSequence[
        subscription_resource.Subscription
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=subscription_resource.Subscription,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class UpdateSubscriptionMetadata(proto.Message):
    r"""Metadata for UpdateSubscription LRO."""


class CreateSubscriptionMetadata(proto.Message):
    r"""Metadata for CreateSubscription LRO."""


class DeleteSubscriptionMetadata(proto.Message):
    r"""Metadata for DeleteSubscription LRO."""


class ReactivateSubscriptionMetadata(proto.Message):
    r"""Metadata for ReactivateSubscription LRO."""


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from .subscription_resource import NotificationEndpoint, PayloadOptions, Subscription
from .subscriptions_service import (
    CreateSubscriptionMetadata,
    CreateSubscriptionRequest,
    DeleteSubscriptionMetadata,
    DeleteSubscriptionRequest,
    GetSubscriptionRequest,
    ListSubscriptionsRequest,
    ListSubscriptionsResponse,
    ReactivateSubscriptionMetadata,
    ReactivateSubscriptionRequest,
    UpdateSubscriptionMetadata,
    UpdateSubscriptionRequest,
)

__all__ = (
    "NotificationEndpoint",
    "PayloadOptions",
    "Subscription",
    "CreateSubscriptionMetadata",
    "CreateSubscriptionRequest",
    "DeleteSubscriptionMetadata",
    "DeleteSubscriptionRequest",
    "GetSubscriptionRequest",
    "ListSubscriptionsRequest",
    "ListSubscriptionsResponse",
    "ReactivateSubscriptionMetadata",
    "ReactivateSubscriptionRequest",
    "UpdateSubscriptionMetadata",
    "UpdateSubscriptionRequest",
)



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import duration_pb2  # type: ignore
from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.apps.events.subscriptions.v1",
    manifest={
        "Subscription",
        "PayloadOptions",
        "NotificationEndpoint",
    },
)


class Subscription(proto.Message):
    r"""A subscription to receive events about a Google Workspace resource.
    To learn more about subscriptions, see the `Google Workspace Events
    API overview <https://developers.google.com/workspace/events>`__.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        expire_time (google.protobuf.timestamp_pb2.Timestamp):
            Non-empty default. The timestamp in UTC when
            the subscription expires. Always displayed on
            output, regardless of what was used on input.

            This field is a member of `oneof`_ ``expiration``.
        ttl (google.protobuf.duration_pb2.Duration):
            Input only. The time-to-live (TTL) or duration for the
            subscription. If unspecified or set to ``0``, uses the
            maximum possible duration.

            This field is a member of `oneof`_ ``expiration``.
        name (str):
            Optional. Immutable. Identifier. Resource name of the
            subscription.

            Format: ``subscriptions/{subscription}``
        uid (str):
            Output only. System-assigned unique
            identifier for the subscription.
        target_resource (str):
            Required. Immutable. The Google Workspace resource that's
            monitored for events, formatted as the `full resource
            name <https://google.aip.dev/122#full-resource-names>`__. To
            learn about target resources and the events that they
            support, see `Supported Google Workspace
            events <https://developers.google.com/workspace/events#supported-events>`__.

            A user can only authorize your app to create one
            subscription for a given target resource. If your app tries
            to create another subscription with the same user
            credentials, the request returns an ``ALREADY_EXISTS``
            error.
        event_types (MutableSequence[str]):
            Required. Immutable. Unordered list. Input for creating a
            subscription. Otherwise, output only. One or more types of
            events to receive about the target resource. Formatted
            according to the CloudEvents specification.

            The supported event types depend on the target resource of
            your subscription. For details, see `Supported Google
            Workspace
            events <https://developers.google.com/workspace/events/guides#supported-events>`__.

            By default, you also receive events about the `lifecycle of
            your
            subscription <https://developers.google.com/workspace/events/guides/events-lifecycle>`__.
            You don't need to specify lifecycle events for this field.

            If you specify an event type that doesn't exist for the
            target resource, the request returns an HTTP
            ``400 Bad Request`` status code.
        payload_options (google.apps.events_subscriptions_v1.types.PayloadOptions):
            Optional. Options about what data to include
            in the event payload. Only supported for Google
            Chat events.
        notification_endpoint (google.apps.events_subscriptions_v1.types.NotificationEndpoint):
            Required. Immutable. The endpoint where the
            subscription delivers events, such as a Pub/Sub
            topic.
        state (google.apps.events_subscriptions_v1.types.Subscription.State):
            Output only. The state of the subscription.
            Determines whether the subscription can receive
            events and deliver them to the notification
            endpoint.
        suspension_reason (google.apps.events_subscriptions_v1.types.Subscription.ErrorType):
            Output only. The error that suspended the subscription.

            To reactivate the subscription, resolve the error and call
            the
            [``ReactivateSubscription``][google.apps.events.subscriptions.v1.SubscriptionsService.ReactivateSubscription]
            method.
        authority (str):
            Output only. The user who authorized the creation of the
            subscription.

            Format: ``users/{user}``

            For Google Workspace users, the ``{user}`` value is the
            ```user.id`` <https://developers.google.com/admin-sdk/directory/reference/rest/v1/users#User.FIELDS.ids>`__
            field from the Directory API.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time when the subscription
            is created.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The last time that the
            subscription is updated.
        reconciling (bool):
            Output only. If ``true``, the subscription is in the process
            of being updated.
        etag (str):
            Optional. This checksum is computed by the
            server based on the value of other fields, and
            might be sent on update requests to ensure the
            client has an up-to-date value before
            proceeding.
    """

    class State(proto.Enum):
        r"""Possible states for the subscription.

        Values:
            STATE_UNSPECIFIED (0):
                Default value. This value is unused.
            ACTIVE (1):
                The subscription is active and can receive
                and deliver events to its notification endpoint.
            SUSPENDED (2):
                The subscription is unable to receive events due to an
                error. To identify the error, see the
                [``suspension_reason``][google.apps.events.subscriptions.v1.Subscription.suspension_reason]
                field.
            DELETED (3):
                The subscription is deleted.
        """
        STATE_UNSPECIFIED = 0
        ACTIVE = 1
        SUSPENDED = 2
        DELETED = 3

    class ErrorType(proto.Enum):
        r"""Possible errors for a subscription.

        Values:
            ERROR_TYPE_UNSPECIFIED (0):
                Default value. This value is unused.
            USER_SCOPE_REVOKED (1):
                The authorizing user has revoked the grant of one or more
                OAuth scopes. To learn more about authorization for Google
                Workspace, see `Configure the OAuth consent
                screen <https://developers.google.com/workspace/guides/configure-oauth-consent#choose-scopes>`__.
            RESOURCE_DELETED (2):
                The target resource for the subscription no
                longer exists.
            USER_AUTHORIZATION_FAILURE (3):
                The user that authorized the creation of the
                subscription no longer has access to the
                subscription's target resource.
            ENDPOINT_PERMISSION_DENIED (4):
                The Google Workspace application doesn't have
                access to deliver events to your subscription's
                notification endpoint.
            ENDPOINT_NOT_FOUND (6):
                The subscription's notification endpoint
                doesn't exist, or the endpoint can't be found in
                the Google Cloud project where you created the
                subscription.
            ENDPOINT_RESOURCE_EXHAUSTED (7):
                The subscription's notification endpoint
                failed to receive events due to insufficient
                quota or reaching rate limiting.
            OTHER (5):
                An unidentified error has occurred.
        """
        ERROR_TYPE_UNSPECIFIED = 0
        USER_SCOPE_REVOKED = 1
        RESOURCE_DELETED = 2
        USER_AUTHORIZATION_FAILURE = 3
        ENDPOINT_PERMISSION_DENIED = 4
        ENDPOINT_NOT_FOUND = 6
        ENDPOINT_RESOURCE_EXHAUSTED = 7
        OTHER = 5

    expire_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=13,
        oneof="expiration",
        message=timestamp_pb2.Timestamp,
    )
    ttl: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=14,
        oneof="expiration",
        message=duration_pb2.Duration,
    )
    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    uid: str = proto.Field(
        proto.STRING,
        number=2,
    )
    target_resource: str = proto.Field(
        proto.STRING,
        number=4,
    )
    event_types: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=5,
    )
    payload_options: "PayloadOptions" = proto.Field(
        proto.MESSAGE,
        number=6,
        message="PayloadOptions",
    )
    notification_endpoint: "NotificationEndpoint" = proto.Field(
        proto.MESSAGE,
        number=7,
        message="NotificationEndpoint",
    )
    state: State = proto.Field(
        proto.ENUM,
        number=8,
        enum=State,
    )
    suspension_reason: ErrorType = proto.Field(
        proto.ENUM,
        number=18,
        enum=ErrorType,
    )
    authority: str = proto.Field(
        proto.STRING,
        number=10,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=11,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=12,
        message=timestamp_pb2.Timestamp,
    )
    reconciling: bool = proto.Field(
        proto.BOOL,
        number=15,
    )
    etag: str = proto.Field(
        proto.STRING,
        number=17,
    )


class PayloadOptions(proto.Message):
    r"""Options about what data to include in the event payload. Only
    supported for Google Chat events.

    Attributes:
        include_resource (bool):
            Optional. Whether the event payload includes data about the
            resource that changed. For example, for an event where a
            Google Chat message was created, whether the payload
            contains data about the
            ```Message`` <https://developers.google.com/chat/api/reference/rest/v1/spaces.messages>`__
            resource. If false, the event payload only includes the name
            of the changed resource.
        field_mask (google.protobuf.field_mask_pb2.FieldMask):
            Optional. If ``include_resource`` is set to ``true``, the
            list of fields to include in the event payload. Separate
            fields with a comma. For example, to include a Google Chat
            message's sender and create time, enter
            ``message.sender,message.createTime``. If omitted, the
            payload includes all fields for the resource.

            If you specify a field that doesn't exist for the resource,
            the system ignores the field.
    """

    include_resource: bool = proto.Field(
        proto.BOOL,
        number=1,
    )
    field_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


class NotificationEndpoint(proto.Message):
    r"""The endpoint where the subscription delivers events.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        pubsub_topic (str):
            Immutable. The Cloud Pub/Sub topic that receives events for
            the subscription.

            Format: ``projects/{project}/topics/{topic}``

            You must create the topic in the same Google Cloud project
            where you create this subscription.

            When the topic receives events, the events are encoded as
            Cloud Pub/Sub messages. For details, see the `Google Cloud
            Pub/Sub Protocol Binding for
            CloudEvents <https://github.com/googleapis/google-cloudevents/blob/main/docs/spec/pubsub.md>`__.

            This field is a member of `oneof`_ ``endpoint``.
    """

    pubsub_topic: str = proto.Field(
        proto.STRING,
        number=1,
        oneof="endpoint",
    )


__all__ = tuple(sorted(__protobuf__.manifest))
